@@ -8,6 +8,7 @@ import traceback
 from codyer import skills
 import uuid
 from codyer_agent.file_operator import FILE_OPERATOR_PROMPT, parse_llm_file_operate
+from codyer_agent.message_compress import COMPRESS_PROMPT
 
 def general_llm_token_count(messages):
     # 统一token计算方式
@@ -402,13 +403,34 @@ def default_get_file_memory(messages: list, file_paths: list, left_token_count:
     """
     file_memory = ''
     for file_path in file_paths:
-        file_memory += f"## {file_path}\n"
-        with open(file_path, 'r', encoding='utf-8') as f:
-            file_memory += f.read() + '\n'
+        try:
+            with open(file_path, 'r', encoding='utf-8') as f:
+                content = f.read()
+                lines = content.splitlines()
+                total_lines = len(lines)
+                
+                file_memory += f"%%FILE_CONTENT%%\n"
+                file_memory += f"{file_path}\n"
+                file_memory += f"lines={total_lines}\n"
+                file_memory += f"start_line=1\n"
+                file_memory += f"end_line={total_lines}\n"
+                file_memory += f"%%CONTENT%%\n"
+                
+                for i, line in enumerate(lines, 1):
+                    file_memory += f"{i}: {line}\n"
+                
+                file_memory += f"%%END%%\n\n"
+        except Exception as e:
+            file_memory += f"%%FILE_CONTENT%%\n"
+            file_memory += f"{file_path}\n"
+            file_memory += f"Error reading file: {str(e)}\n"
+            file_memory += f"%%END%%\n\n"
+    
     return file_memory
 
 def default_get_agent_memory(original_messages: list, workspace= '', task=''):
     return original_messages
+    
         
 class Agent:
     python_prompt_template = defalut_python_prompt_template
@@ -421,15 +443,15 @@ class Agent:
             model=None,
             llm_inference=skills.codyer.llm.llm_inference,
             llm_token_count=general_llm_token_count,
-            llm_token_limit=64000,
+            llm_token_limit=100000,
             continue_run=False,
             messages=None,
             enable_python=True,
             max_steps=5, # 最大re act次数
             interpretors = [], # 解析器 [is_match, parse, 'realtime' or 'final']
             with_file_operator=False,
-            get_file_memory=skills.codyer.llm.get_file_memory,
-            get_agent_memory=skills.codyer.llm.get_agent_memory,
+            get_file_memory=default_get_file_memory, # skills.codyer.llm.get_file_memory
+            get_agent_memory=default_get_agent_memory, # skills.codyer.llm.get_agent_memory，已经没有用了
             read_files = [], # 参考文件列表，只能读取.md、txt、py、js等文本文件
             show_step_detail=False, # 是否显示每一步的执行信息细节
         ):
@@ -439,7 +461,7 @@ class Agent:
         @workspace: str, agent保存记忆的工作空间，默认值为None（不序列化）。如果指定了目录，Agent会自动保存状态并在下次初始化时重新加载。
         @stream_output: function, agent输出回调函数
         @llm_inference: function, LLM推理函数
-        @llm_token_limit: int, LLM token limit, default 64000
+        @llm_token_limit: int, LLM token limit, default 100K
         @continue_run: bool, 是否自动继续执行。Agent在任务没有完成时，是否自动执行。默认为True. 该参数已经废弃，请使用 max_steps 参数代替
         @messages: list, agent记忆 [{"role": "system", "content": "You are a helpful assistant"}, {"role": "user", "content": str | ['text', {'image': 'image_url | image_path']}]
         @enable_python: bool, 是否启用agent执行python代码和调用functions
@@ -490,11 +512,58 @@ class Agent:
         self.llm_run_count = 0
         self._with_file_operator = with_file_operator
         if self._with_file_operator:
-            self._interpretors = [(self.is_math_file_operator, self.parse_file_operator, 'final')] + self._interpretors # 先执行文件操作
+            # 先执行读文件操作（实时），再执行其他文件操作
+            self._interpretors = [
+                (self.is_file_read_operator, self.parse_file_read_operator, 'realtime'),  # 实时读文件
+                (self.is_file_write_operator, self.parse_file_write_operator, 'final'),   # 其他文件操作
+            ] + self._interpretors
         self._read_files = read_files
         self._tmp_read_files = []
         self._show_step_detail = show_step_detail
 
+    def is_file_read_operator(self, llm_output):
+        """检查是否包含读文件操作"""
+        if '%%FILE_OPERATION%%' not in llm_output:
+            return False
+            
+        try:
+            from .file_operator import _find_all_file_operations, _parse_file_operation_format
+            operations = _find_all_file_operations(llm_output)
+            
+            for operation_text in operations:
+                try:
+                    operation_data = _parse_file_operation_format(operation_text)
+                    if operation_data["operation"] == "read":
+                        return True
+                except:
+                    continue
+            return False
+        except:
+            return False
+            
+    def parse_file_read_operator(self, llm_output):
+        """解析读文件操作（实时）"""
+        from .file_operator import parse_llm_file_read
+        result = parse_llm_file_read(llm_output)
+        return False, None, result, True
+    
+    def is_file_write_operator(self, llm_output):
+        """检查是否包含写文件操作（除读取外的其他操作）"""
+        if '%%FILE_OPERATION%%' in llm_output:
+            # 检查是否包含非读取的操作
+            operations = ['write ', 'append ', 'update ', 'delete_range ', 'delete_all ']
+            for op in operations:
+                if op in llm_output:
+                    return True
+        return False
+        
+    def parse_file_write_operator(self, llm_output):
+        """解析写文件等操作（非读取操作）"""
+        from .file_operator import parse_llm_file_operate_no_read
+        result = parse_llm_file_operate_no_read(llm_output)
+        return False, None, result, True
+
+
     def is_math_file_operator(self, llm_output):
         if '%%FILE_OPERATION%%' in llm_output:
             return True
@@ -510,6 +579,7 @@ class Agent:
         import datetime
         # 加载最新消息
         load_messages = self.load_messages()
+        self.messages = []
         if load_messages is not None and len(load_messages) > 0:
             self.messages = load_messages
         if isinstance(content, list):
@@ -525,8 +595,37 @@ class Agent:
                 for i, x in enumerate(content):
                     if isinstance(x, str):
                         content[i] = f"[{current_time}] {x}"
-            self.messages = self.get_agent_memory(original_messages=self.messages, workspace=self.workspace, task=str(content))
+
+        # 压缩消息
+        token_count = int(self.llm_token_count(self.messages))
+        # if self._show_step_detail:
+        #     self.stream_output(f"\n上下文长度: {token_count}")
+        if token_count > self.llm_token_limit * 2 / 3:
+            if self._show_step_detail:
+                self.stream_output(f"\n```memory\n上下文长度: {token_count}，进行操作 \n")
+            # 压缩前面的 80% 内容，避免暴力压缩了全部内容
+            compress_count = int(token_count * 0.82)
+            compress_messages = []
+            for index, message in enumerate(self.messages):
+                compress_messages.append(message)
+                if self.llm_token_count(compress_messages) > compress_count:
+                    break
+            left_messages = self.messages[index:]
+            # if self._show_step_detail:
+            #     self.stream_output(f'压缩消息条数: {len(compress_messages)}')
+            #     self.stream_output(f'剩余消息条数: {len(left_messages)}')
+            summary = self._compress_messages(compress_messages)
+            if len(left_messages) > 0:
+                the_role = 'assistant' if left_messages[0]['role'] == 'user' else 'user' # 压缩消息后，和下一条消息的角色保持区分，避免消息合并
+            else:
+                the_role = 'assistant' if role == 'user' else 'user' # 压缩消息后，和下一条消息的角色保持区分，避免消息合并
+            self.messages = [{"role": the_role, "content": summary}] + left_messages
+            if self._show_step_detail:
+                token_count = self.llm_token_count(self.messages)
+                self.stream_output(f'\n处理后长度为 {token_count}\n```\n')
+            #     self.stream_output(f'压缩后内容: \n---------\n{summary}\n---------\n')
         
+        # 添加消息
         if self.messages is not None and len(self.messages) > 0 and self.messages[-1]["role"] == role:
             if isinstance(self.messages[-1]["content"], str) and isinstance(content, str):
                 self.messages[-1]["content"] += '\n\n' + content
@@ -711,6 +810,19 @@ class Agent:
         return True
     
     def _run(self, input, is_run_mode=False, return_type=None):
+
+        # 整理系统提示词
+        if not self._enable_python:
+            self.system_prompt = self.role
+        else:
+            funtion_signatures = "\n\n".join([get_function_signature(x) for x in self.functions])
+            variables = {"python_funcs": funtion_signatures}
+            python_prompt = Template(self.python_prompt_template).render(**variables)
+            self.system_prompt = self.role + '\n\n' + python_prompt
+        if self._with_file_operator:
+            self.system_prompt += '\n\n对任何文件进行操作必须使用以下指令：' + FILE_OPERATOR_PROMPT
+        self.system_prompt += '\n\nAfter completing the user\'s request, please output <<terminate>> to end the current task.'
+    
         # 如果是run模式 & 需要返回值类型
         if is_run_mode and return_type is not None:
             add_content = "\nYou should return python values in type " + str(return_type) + " by run python code(```python\n# run code\nxxx\n).\n"
@@ -738,6 +850,9 @@ class Agent:
                     return llm_result + "\n任务结束: 任务完成 或 进行循环 或 目标偏离"
                 
             messages = self._get_llm_messages()
+            if self._show_step_detail:
+                self.stream_output(f'\n全部消息: {str(self.messages)}\n')
+            
             llm_result, (python_mode, python_data, result, should_continue) = self._llm_and_parse_output(messages)
             self.llm_run_count += 1
             if is_run_mode and python_mode:
@@ -772,52 +887,32 @@ class Agent:
         
         # 循环结束时返回最后的结果
         return llm_result + "已达到最大执行步数"
-
-    def _cut_messages(self, messages, llm_token_limit):
-        """剪切消息"""
-        # 最近的一条user消息不能压缩
-        # 其他消息都可以压缩，压缩有格式为: '<detail deleted> Summary:\n xxxx'
-        # 最近一条用户输入位置
-        last_user_index = 0
-        for index in range(len(messages)):
-            if messages[index]['role'] == 'user':
-                last_user_index = index
-        
-        # 进行压缩
-        if self.llm_token_count(messages) > llm_token_limit:
-            for index in range(len(messages)):
-                compressed = False
-                if index != last_user_index:
-                    if isinstance(messages[index]['content'], str):
-                        if not messages[index]['content'].startswith('<detail deleted>'):
-                            messages[index]['content'] = self._compress_message(messages[index]['content'])
-                            compressed = True
-                    elif isinstance(messages[index]['content'], list):
-                        for x, item in enumerate(messages[index]['content']):
-                            if isinstance(item, str):
-                                if not item.startswith('<detail deleted>'):
-                                    item = self._compress_message(item)
-                                    messages[index]['content'][x] = item
-                                    compressed = True
-                if compressed:
-                    if self.llm_token_count(messages) < llm_token_limit:
-                        break
-        
-        # 直接进行裁剪
-        while self.llm_token_count(messages) > llm_token_limit:
-            messages.pop(0)
-
-        return messages
     
-    def _compress_message(self, message):
-        prompt = message + '\n\n简单的一段话概要上面的内容，且包含了最核心的一些关键词，如姓名、函数或者类名、核心技术名词、故事冲突点等。'
+    def _compress_messages(self, messages):
+        """压缩消息"""
         for index, llm_inference in enumerate(self.llm_inferences):
             try:
-                response = llm_inference([{'role': 'user', 'content': prompt}], stream=False)
-                return '<detail deleted> Summary:\n' + response
-            except:
-                pass
-        return None
+                # response = llm_inference([{"role": "system", "content": '你是一个消息压缩专家，对下面的内容进行压缩'}] + messages + [{'role': 'user', 'content': COMPRESS_PROMPT}], stream=False)
+                # return response
+                response = llm_inference([{"role": "system", "content": '你是一个消息压缩专家，对下面的内容进行压缩'}] + messages + [{'role': 'user', 'content': COMPRESS_PROMPT}], stream=True)
+                result = ''
+                for x in response:
+                    # print(x)
+                    result += x
+                return result
+            except Exception as e:
+                print(str(e))
+        raise ValueError('Agent消息压缩出错')
+    
+    # def _compress_message(self, message):
+    #     prompt = message + '\n\n简单的一段话概要上面的内容，且包含了最核心的一些关键词，如姓名、函数或者类名、核心技术名词、故事冲突点等。'
+    #     for index, llm_inference in enumerate(self.llm_inferences):
+    #         try:
+    #             response = llm_inference([{'role': 'user', 'content': prompt}], stream=False)
+    #             return '<detail deleted> Summary:\n' + response
+    #         except:
+    #             pass
+    #     return None
 
 
     def _get_llm_messages(self):
@@ -826,29 +921,10 @@ class Agent:
         @return: list, [{"role": "system|assistant|user", "content": "content"}, ...]
         """
         # 长度分配: 整体长度小于80%的token限制，去掉SYSTEM PROMPT后，剩余的token平均分配给MESSAGES和FILES
-        files = self._read_files + self._tmp_read_files
-        has_file = len(files) > 0
-        # SYSTEM PROMPT
-        messages = self.messages
-        if not self._enable_python:
-            system_prompt = self.role
-        else:
-            funtion_signatures = "\n\n".join([get_function_signature(x) for x in self.functions])
-            variables = {"python_funcs": funtion_signatures}
-            python_prompt = Template(self.python_prompt_template).render(**variables)
-            system_prompt = self.role + '\n\n' + python_prompt
-        if self._with_file_operator:
-            system_prompt += '\n\n对任何文件进行操作必须使用以下指令：' + FILE_OPERATOR_PROMPT
-        system_prompt += '\n\nAfter completing the user\'s request, please output <<terminate>> to end the current task.'
         
-        # MESSAGES
-        # system_prompt_count = self.llm_token_count([{"role": "system", "content": system_prompt}])
-        # left_count = int(self.llm_token_limit * 0.8) - system_prompt_count
-        # message_token_count = left_count if not has_file else int(left_count / 2)
-        # messages = self._cut_messages(messages, message_token_count)
         # 合并messages中，同类型的连续消息
-        new_messages = [{"role": "system", "content": system_prompt}]
-        for message in messages:
+        new_messages = [{"role": "system", "content": self.system_prompt}]
+        for message in self.messages:
             if new_messages[-1]["role"] != message["role"]:
                 new_messages.append(message)
             else:
@@ -856,20 +932,17 @@ class Agent:
                     new_messages[-1]["content"] += '\n\n' + message["content"]
                 else:
                     new_messages.append(message)
-        self.messages = new_messages[1:]  # 去掉system消息，只保存用户和助手的对话
+        self.messages = new_messages[1:]
         self.save_messages()
-        # FILES
+
+        # 添加文件内容
         files = self._read_files + self._tmp_read_files
-        file_memory = ''
         if len(files) > 0:
-            left_token_count = self.llm_token_limit - self.llm_token_count(new_messages)
-            file_memory = self.get_file_memory(new_messages, files, left_token_count)
-        # 组合messages
-        if file_memory:
-            messages = [{"role": "system", "content": system_prompt + file_memory}] + [{"role": "assistant", "content": file_memory}] + self.messages
-        else:
-            messages = [{"role": "system", "content": system_prompt}] + self.messages
-        return messages
+            file_memory = self.get_file_memory([], files, -1)
+            new_messages += [{"role": "assistant", "content": file_memory}]
+
+        # 添加system消息
+        return new_messages
     
     def _run_python_match(self, result):
         # 检测是否有python代码需要运行
